'use strict';
const Knex = require('knex');

const KnexHelper = {

  /**
   * Provides knex instance.
   * @param {string} database - Database to connect to.
   */
  getKnex(database) {
    let connection = {
      host: 'postgres',
      user: 'vello',
      password: 'admin123'
    };
    if(database) {
      connection.database = database;
    }
    let knex = Knex({
      client: 'postgresql',
      connection: connection
    });
    return knex;
  },

  /**
   * Creates database.
   * @param {string} database - Database to create.
   */
  create(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex();
      KnexHelper.exists(database).then((exists) => {
        if(!exists) {
          knex.raw('CREATE DATABASE ' + database).then(() => {
            console.log('Database: Created database ' + database);
            resolve();
          })
          .catch((error) => {
            console.error(error);
            reject();
          });
        } else {
          console.log('Database: ' + database + ' already exists');
          resolve();
        }
      });
    });
  },

  /**
   * Drops database.
   * @param {string} database - Database to drop.
   */
  drop(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex();
      KnexHelper.exists(database).then((exists) => {
        if(exists) {
          knex.raw('DROP DATABASE ' + database).then(() => {
            console.log('Database: Dropped database ' + database);
            resolve();
          })
          .catch((error) => {
            console.error(error);
            reject();
          });
        } else {
          console.log('Database: ' + database + ' doesn\'t exist');
          resolve();
        }
      });
    });
  },

  /**
   * Checks if database exists.
   * @param {string} database - Database to check.
   */
  exists(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex();
      knex.raw('SELECT datname FROM pg_catalog.pg_database WHERE lower(datname) = lower(\'' + database + '\')')
      .then((results) => {
        resolve(results.rowCount);
      })
      .catch((error) => {
        console.log(error);
      });
    });
  },

  /**
   * Runs latest migration on database.
   * @param {string} database - Database to run migration on.
   */
  migrate(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex(database);
      knex.migrate.latest({
        directory: './migrations'
      })
      .then(() => {
        console.log('Database: Ran migration on ' + database);
        resolve();
      })
      .catch((error) => {
        console.log(error);
      });
    });
  },

  /**
   * Rollsback last migration on database.
   * @param {string} database - Database to rollback.
   */
  rollback(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex(database);
      knex.migrate.rollback({
        directory: './migrations'
      })
      .then(() => {
        console.log('Database: Ran rollback on ' + database);
        resolve();
      })
      .catch((error) => {
        console.log(error);
      });
    });
  },

  /**
   * Runs seeder on database.
   * @param {string} database - Database to seed.
   */
  seed(database) {
    return new Promise((resolve, reject) => {
      let knex = KnexHelper.getKnex(database);
      knex.seed.run({
        directory: './database/seeds'
      })
      .then(() => {
        console.log('Database: Ran seed on ' + database);
        resolve();
      })
      .catch((error) => {
        console.log(error);
      });
    });
  },

};

module.exports = KnexHelper;