exports.up = function(knex) {
  return knex.raw('CREATE EXTENSION IF NOT EXISTS "uuid-ossp"')
  .then(function() {
    return knex.schema

    /**
     * User
     *
     */
    .createTable('user', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('email').notNullable().unique();
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })

    /**
     * Tag
     *
     */
    .createTable('tag', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('name').notNullable().unique();
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })

    /**
     * Media
     *
     */
    .createTable('media', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.enu('type', ['image', 'video']).notNullable();
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })

    /**
     * Post
     *
     */
    .createTable('post', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('title').notNullable();
      table.uuid('media_id').references('media.id').onDelete('CASCADE');
      table.uuid('user_id').notNullable().references('user.id').onDelete('CASCADE');
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })
    /*
    .createTable('post_item', function(table) {
      table.uuid('id').primary().defaultTo(knex.raw('uuid_generate_v4()'));
      table.string('title').notNullable();
      table.string('content');
      table.string('button_label');
      table.string('button_url');
      table.uuid('media_id').references('media.id').onDelete('CASCADE');
      table.uuid('post_id').notNullable().references('post.id').onDelete('CASCADE');
      table.dateTime('created_at').defaultTo(knex.fn.now());
      table.dateTime('updated_at').defaultTo(knex.fn.now());
    })
    */

    /**
     * Many to Many
     *
     */
    .createTable('post_tag', function(table) {
      table.uuid('post_id').references('post.id').onDelete('CASCADE');
      table.uuid('tag_id').references('tag.id').onDelete('CASCADE');
    })
    /*
    .createTable('user_tag', function(table) {
      table.uuid('user_id').references('user.id').onDelete('CASCADE');
      table.uuid('tag_id').references('tag.id').onDelete('CASCADE');
    })
    */

  });
};

exports.down = function(knex) {
  return knex.schema
  .dropTable('post_tag')
  //.dropTable('user_tag')
  //.dropTable('post_item')
  .dropTable('post')
  .dropTable('media')
  .dropTable('tag')
  .dropTable('user');
};
